const fs = require('fs');
const path = require('path');

const config = require(process.cwd() + '/package.json').config;
var stopDirectories = ['.git', 'node_modules', 'cartridge'];

var traverse = function (base, ext, files, result) {
    base = base || './';
    ext = ext || 'project';
    files = files || fs.readdirSync(base);
    result = result || {}; // {}, [] || Set();

    // console.log(files);
    files.forEach((file, idx, files) => {
        if (stopDirectories.includes(file)) {
            return;
        }
        
        base = path.relative(process.cwd(), base);

        var newbase = path.join(base, file);
        // console.log(`newbase: ${newbase}`);
        if (fs.statSync(newbase).isDirectory()) {
            result = traverse(newbase, ext, fs.readdirSync(newbase), result);
        }
        else {
            // console.log(`file: ${file}`);
            if (file.substr(-1 * (ext.length + 1)) == '.' + ext) {
                let key = base.substring(base.lastIndexOf(path.sep) + 1);
                // result.push({[key]:base});
                result[key] = {
                    alias: '',
                    data: '',
                    name: key,
                    path: base
                };

                // var siblings = arguments[2] || files;
                if (files.includes('site-import')) {
                    result[key].data = path.join(base, 'site-import');
                }
            }
        }
    });
    
    return result
};
module.exports.findAllCartridges = traverse;

/**
 * search directories
 * @param {Array} directories 
 * @returns {Object}
 */
var searchDirectories = function (directories) {
    if (!directories || !Array.isArray(directories) || !directories.length) {
        directories = config && config.code && config.code.directories;
    }

    var locatedCartridges = {};
    directories.forEach(directory => {
        try {
            let cartridges = traverse(directory);
            locatedCartridges = Object.assign(locatedCartridges, cartridges);
        }
        catch (e) {
            return;
        }
    });
    
    if (!Object.keys(locatedCartridges).length) {
        console.error(`no cartridges located within directories: ${JSON.stringify(directories, null, 1)}`);
        return [];
    }

    return Object.keys(locatedCartridges).map(cartridge => {
        return locatedCartridges[cartridge];
    });
}
module.exports.searchDirectories = searchDirectories;

/**`
 * 
 * @returns Array
 */
module.exports.getFilterList = function () {
    try {
        var cartridgesJson = require(process.cwd() + '/cartridges.json');
    }
    catch (e) {
        cartridgesJson = null;
        console.log(`\n\nno cartridges.json located.\n\n`)
    }
    return cartridgesJson;
}

module.exports.filterCartridges = function () {
    var filterList = this.getFilterList();
    if (!filterList || !Array.isArray(filterList) || !filterList.length) {
        return [];
    }

    console.log(`\n\nlocated cartridges.json\n\n`);

    var allCartridges = traverse();
    var filteredList = [];
    filterList.forEach(cartridge => {
        if (allCartridges[cartridge]) {
            filteredList.push(allCartridges[cartridge]);
        }
    });
    return filteredList;
}

module.exports.printIncludedCartridges = function (cartridges) {
    if (!cartridges || !Array.isArray(cartridges) || !cartridges.length) {
        return console.log('empty list passed to get included cartridges method');
    }
    var includedList = cartridges.map(cartridge => {
        return cartridge.name;
    });

    console.log(`cartridges included within deployment:\n${JSON.stringify(includedList, null, 1)} `);
}

module.exports.getAllCartridges = function getAllCartridges () {
    var dir = ['./'];
    return this.searchDirectories(dir);
}

module.exports.getCartridges = function () {
    var cartridges = this.filterCartridges();
    
    if (!cartridges || !cartridges.length) {
        console.log(`attempting to use config.code.directories object within package.json\n`);
        cartridges = this.searchDirectories();
    }

    if (!cartridges || !cartridges.length) {
        console.log(`\n\nattempting to include all cartridges within project.\n`);
        cartridges = this.getAllCartridges();
    }

    return cartridges;
}
